@@ -1,7 +1,6 @@
-#!C:\bld\qds-sdk_1491653682897\_b_env\python.exe
+#!C:\bld\qds-sdk_1498449821918\_b_env\python.exe
 
 from __future__ import print_function
-from qds_sdk.qubole import Qubole
 from qds_sdk.commands import *
 from qds_sdk.cluster import *
 import qds_sdk.exception
@@ -16,7 +15,8 @@ from qds_sdk.app import AppCmdLine
 from qds_sdk.nezha import NezhaCmdLine
 from qds_sdk.user import UserCmdLine
 from qds_sdk.template import TemplateCmdLine
-
+from qds_sdk.clusterv2 import ClusterCmdLine
+from qds_sdk.sensors import *
 import os
 import sys
 import traceback
@@ -37,15 +37,23 @@ CommandClasses = {
     "prestocmd": PrestoCommand
 }
 
+SensorClasses = {
+    "filesensor": FileSensor,
+    "partitionsensor": PartitionSensor
+}
+
 usage_str = (
     "Usage: qds.py [options] <subcommand>\n"
     "\nCommand subcommands:\n"
     "  <hivecmd|hadoopcmd|prestocmd|pigcmd|shellcmd|dbexportcmd|dbimportcmd|dbtapquerycmd|sparkcmd> <action>\n"
     "    submit [cmd-specific-args .. ] : submit cmd & print id\n"
     "    run [cmd-specific-args .. ] : submit cmd & wait. print results\n"
-    "    check <id> : print the cmd object for this id\n"
+    "    check <id> <include-query-properties> : id -> print the cmd object for this id\n"
+    "                                            include-query-properties(true/false) -> to include query properties like\n"
+    "                                            tags, comments and user actions\n"
     "    cancel <id> : cancels the cmd with this id\n"
-    "    getresult <id> : get the results for the cmd with this id\n"
+    "    getresult <id> <include_header>: id -> get the results for the cmd with this id\n"
+    "                                     include_header -> to include headers in results(true/false)\n"
     "    getlog <id> : get the logs for the cmd with this id\n"
     "\nCluster subcommand:\n"
     "  cluster <action>\n"
@@ -86,7 +94,9 @@ usage_str = (
     "\nNezha subcommand:\n"
     "  nezha --help\n"
     "\nUser subcommad:\n"
-    "  user --help\n")
+    "  user --help\n"
+    "\nSensor subcommand:\n"
+    " <filesensor|partitionsensor> --help\n")
 
 
 def usage(parser=None):
@@ -113,10 +123,10 @@ def submitaction(cmdclass, args):
         return 0
 
 
-def _getresult(cmdclass, cmd):
+def _getresult(cmdclass, cmd, args=[]):
     if Command.is_success(cmd.status):
         log.info("Fetching results for %s, Id: %s" % (cmdclass.__name__, cmd.id))
-        cmd.get_results(sys.stdout, delim='\t')
+        cmd.get_results(sys.stdout, delim='\t', qlog=cmd.qlog, arguments=args)
         return 0
     else:
         log.error("Cannot fetch results - command Id: %s failed with status: %s" % (cmd.id, cmd.status))
@@ -134,9 +144,20 @@ def runaction(cmdclass, args):
 
 
 def checkaction(cmdclass, args):
-    checkargs_id(args)
-    o = cmdclass.find(args.pop(0))
-    print(str(o))
+    if len(args) > 2:
+        sys.stderr.write("expecting not more than 2 arguments\n")
+        usage()
+
+    conn = Qubole.agent()
+    id = args.pop(0)
+    include_query_properties="false"
+    if len(args) == 1:
+        include_query_properties=args.pop(0)
+        if include_query_properties not in ('true', 'false'):
+            raise ParseError("include-query-properties can be either true or false")
+
+    r = conn.get(cmdclass.element_path(id), {'include_query_properties': include_query_properties})
+    print(str(r))
     return 0
 
 
@@ -156,9 +177,12 @@ def cancelaction(cmdclass, args):
 
 
 def getresultaction(cmdclass, args):
-    checkargs_id(args)
+    if len(args) > 2:
+        sys.stderr.write("expecting not more than 2 arguments\n")
+        usage()
+
     cmd = cmdclass.find(args.pop(0))
-    return _getresult(cmdclass, cmd)
+    return _getresult(cmdclass, cmd, args)
 
 
 def getlogaction(cmdclass, args):
@@ -195,6 +219,12 @@ def cmdmain(cmd, args):
     return globals()[action + "action"](cmdclass, args)
 
 
+def sensormain(sensor, args):
+    sensor_class = SensorClasses[sensor]
+    print(SensorCmdLine.check(sensor_class, args))
+    return 0
+
+
 def checkargs_cluster_id_label(args):
     if len(args) != 1:
         sys.stderr.write("expecting single argument cluster id or cluster label\n")
@@ -208,7 +238,6 @@ def cluster_create_action(clusterclass, args, api_version=1.2):
     print(json.dumps(result, indent=4))
     return 0
 
-
 def cluster_update_action(clusterclass, args, api_version=1.2):
     arguments = clusterclass._parse_create_update(args, "update", api_version)
     cluster_info = _create_cluster_info(arguments, api_version)
@@ -238,6 +267,7 @@ def _create_cluster_info(arguments, api_version):
                                       aws_availability_zone=arguments.aws_availability_zone,
                                       vpc_id=arguments.vpc_id,
                                       subnet_id=arguments.subnet_id,
+                                      master_elastic_ip=arguments.master_elastic_ip,
                                       disallow_cluster_termination=arguments.disallow_cluster_termination,
                                       enable_ganglia_monitoring=arguments.enable_ganglia_monitoring,
                                       node_bootstrap_file=arguments.node_bootstrap_file,
@@ -270,7 +300,8 @@ def _create_cluster_info(arguments, api_version):
                                       enable_presto=arguments.enable_presto,
                                       bastion_node_public_dns=arguments.bastion_node_public_dns,
                                       role_instance_profile=arguments.role_instance_profile,
-                                      presto_custom_config=presto_custom_config)
+                                      presto_custom_config=presto_custom_config,
+                                      is_ha=arguments.is_ha)
     else:
         cluster_info = ClusterInfo(arguments.label,
                                    arguments.aws_access_key_id,
@@ -283,6 +314,7 @@ def _create_cluster_info(arguments, api_version):
                                       arguments.aws_availability_zone,
                                       arguments.vpc_id,
                                       arguments.subnet_id,
+                                      arguments.master_elastic_ip,
                                       arguments.role_instance_profile,
                                       arguments.bastion_node_public_dns)
 
@@ -295,7 +327,8 @@ def _create_cluster_info(arguments, api_version):
                                          arguments.use_hbase,
                                          arguments.custom_ec2_tags,
                                          arguments.use_hadoop2,
-                                         arguments.use_spark)
+                                         arguments.use_spark,
+                                         arguments.is_ha)
 
         cluster_info.set_spot_instance_settings(
               arguments.maximum_bid_price_percentage,
@@ -437,6 +470,23 @@ def clustermain(args, api_version):
     else:
         return globals()["cluster_" + action + "_action"](clusterclass, args)
 
+def clustermainv2(args):
+    action = args[0]
+    actionset = set(
+        ["create", "delete", "update", "clone", "list", "start", "terminate", "status", "reassign_label", "add_node",
+         "remove_node", "update_node", "snapshot", "restore_point", "get_snapshot_schedule",
+         "update_snapshot_schedule"])
+
+    result = None
+    if action not in actionset:
+        sys.stderr.write("action must be one of <%s>\n" % "|".join(actionset))
+        usage()
+    elif action in set(["create", "update", "clone"]):
+        result =  ClusterCmdLine.run(args)
+    else:
+        result =  globals()["cluster_" + action + "_action"](Cluster, args)
+    print(result)
+
 def accountmain(args):
     result = AccountCmdLine.run(args)
     print(result)
@@ -481,7 +531,7 @@ def nezhamain(args):
 def templatemain(args):
     result = TemplateCmdLine.run(args)
     print(result)
-    
+
 
 def main():
     optparser = OptionParser(usage=usage_str)
@@ -506,6 +556,10 @@ def main():
                          default=False,
                          help="skip verification of server SSL certificate. Insecure: use with caution.")
 
+    optparser.add_option("--cloud_name", dest="cloud_name",
+                         default=os.getenv('CLOUD_PROVIDER'),
+                         help="cloud", choices=["AWS", "AZURE", "ORACLE_BMC"])
+
     optparser.add_option("-v", dest="verbose", action="store_true",
                          default=False,
                          help="verbose mode - info level logging")
@@ -537,6 +591,9 @@ def main():
     if options.poll_interval is None:
         options.poll_interval = 5
 
+    if options.cloud_name is None:
+        options.cloud_name = "AWS"
+
     if options.skip_ssl_cert_check is None:
         options.skip_ssl_cert_check = False
     elif options.skip_ssl_cert_check:
@@ -546,7 +603,8 @@ def main():
                      api_url=options.api_url,
                      version=options.api_version,
                      poll_interval=options.poll_interval,
-                     skip_ssl_cert_check=options.skip_ssl_cert_check)
+                     skip_ssl_cert_check=options.skip_ssl_cert_check,
+                     cloud_name=options.cloud_name)
 
     if len(args) < 1:
         sys.stderr.write("Missing first argument containing subcommand\n")
@@ -556,12 +614,18 @@ def main():
     if a0 in CommandClasses:
         return cmdmain(a0, args)
 
+    if a0 in SensorClasses:
+        return sensormain(a0, args)
+
     if a0 == "account":
         return accountmain(args)
 
     if a0 == "cluster":
         api_version_number = float(options.api_version[1:])
-        return clustermain(args, api_version_number)
+        if api_version_number >= 2.0:
+            return clustermainv2(args)
+        else:
+            return clustermain(args, api_version_number)
 
     if a0 == "action":
         return actionmain(args)
