@@ -6,6 +6,7 @@ cluster information.
 from qds_sdk.qubole import Qubole
 from qds_sdk.resource import Resource
 from argparse import ArgumentParser
+from qds_sdk import util
 
 import logging
 import json
@@ -171,6 +172,9 @@ class Cluster(Resource):
         ec2_group.add_argument("--vpc-id",
                                dest="vpc_id",
                                help="vpc to create the cluster in",)
+        ec2_group.add_argument("--master-elastic-ip",
+                               dest="master_elastic_ip",
+                               help="elastic ip to attach to master",)
         ec2_group.add_argument("--bastion-node-public-dns",
                                dest="bastion_node_public_dns",
                                help="public dns name of the bastion node. Required only if cluster is in private subnet of a EC2-VPC",)
@@ -210,6 +214,9 @@ class Cluster(Resource):
         hadoop_group.add_argument("--use-hbase", dest="use_hbase",
                                   action="store_true", default=None,
                                   help="Use hbase on this cluster",)
+        hadoop_group.add_argument("--is-ha", dest="is_ha",
+                                  action="store_true", default=None,
+                                  help="Enable HA config for cluster")
         if api_version >= 1.3:
           qubole_placement_policy_group = hadoop_group.add_mutually_exclusive_group()
           qubole_placement_policy_group.add_argument("--use-qubole-placement-policy",
@@ -715,6 +722,7 @@ class ClusterInfo():
                          aws_availability_zone=None,
                          vpc_id=None,
                          subnet_id=None,
+                         master_elastic_ip=None,
                          role_instance_profile=None,
                          bastion_node_public_dns=None):
         """
@@ -737,6 +745,7 @@ class ClusterInfo():
         self.ec2_settings['vpc_id'] = vpc_id
         self.ec2_settings['subnet_id'] = subnet_id
         self.ec2_settings['role_instance_profile'] = role_instance_profile
+        self.ec2_settings['master_elastic_ip'] = master_elastic_ip
         self.ec2_settings['bastion_node_public_dns'] = bastion_node_public_dns
 
     def set_hadoop_settings(self, master_instance_type=None,
@@ -748,7 +757,8 @@ class ClusterInfo():
                             use_hbase=None,
                             custom_ec2_tags=None,
                             use_hadoop2=None,
-                            use_spark=None):
+                            use_spark=None,
+                            is_ha=None):
         """
         Kwargs:
 
@@ -773,6 +783,9 @@ class ClusterInfo():
         `use_hadoop2`: Use hadoop2 in this cluster
 
         `use_spark`: Use spark in this cluster
+
+        `is_ha` : enable HA config for cluster
+
         """
         self.hadoop_settings['master_instance_type'] = master_instance_type
         self.hadoop_settings['slave_instance_type'] = slave_instance_type
@@ -783,6 +796,7 @@ class ClusterInfo():
         self.hadoop_settings['use_hbase'] = use_hbase
         self.hadoop_settings['use_hadoop2'] = use_hadoop2
         self.hadoop_settings['use_spark'] = use_spark
+        self.hadoop_settings['is_ha'] = is_ha
 
         if custom_ec2_tags and custom_ec2_tags.strip():
             try:
@@ -882,7 +896,7 @@ class ClusterInfo():
         creating or updating a cluster.
         """
         payload = {"cluster": self.__dict__}
-        return _make_minimal(payload)
+        return util._make_minimal(payload)
 
 class ClusterInfoV13():
     """
@@ -914,6 +928,7 @@ class ClusterInfoV13():
                          aws_availability_zone=None,
                          vpc_id=None,
                          subnet_id=None,
+                         master_elastic_ip=None,
                          disallow_cluster_termination=None,
                          enable_ganglia_monitoring=None,
                          node_bootstrap_file=None,
@@ -946,7 +961,8 @@ class ClusterInfoV13():
                          enable_presto=None,
                          bastion_node_public_dns=None,
                          role_instance_profile=None,
-                         presto_custom_config=None):
+                         presto_custom_config=None,
+                         is_ha=None):
         """
         Kwargs:
 
@@ -965,6 +981,8 @@ class ClusterInfoV13():
 
         `subnet_id`: The subnet to create the cluster in.
 
+        `master_elastic_ip`: Elastic IP to attach to master node
+
         `disallow_cluster_termination`: Set this to True if you don't want
             qubole to auto-terminate idle clusters. Use this option with
             extreme caution.
@@ -1052,6 +1070,8 @@ class ClusterInfoV13():
 
         `bastion_node_public_dns`: Public dns name of the bastion node. Required only if cluster is in private subnet.
 
+        `is_ha`: Enabling HA config for cluster
+
         """
 
         self.disallow_cluster_termination = disallow_cluster_termination
@@ -1059,8 +1079,8 @@ class ClusterInfoV13():
         self.node_bootstrap_file = node_bootstrap_file
         self.set_node_configuration(master_instance_type, slave_instance_type, initial_nodes, max_nodes, slave_request_type, fallback_to_ondemand)
         self.set_ec2_settings(aws_access_key_id, aws_secret_access_key, aws_region, aws_availability_zone, vpc_id, subnet_id,
-                                bastion_node_public_dns, role_instance_profile)
-        self.set_hadoop_settings(custom_config, use_hbase, custom_ec2_tags, use_hadoop2, use_spark, use_qubole_placement_policy)
+                              master_elastic_ip, bastion_node_public_dns, role_instance_profile)
+        self.set_hadoop_settings(custom_config, use_hbase, custom_ec2_tags, use_hadoop2, use_spark, use_qubole_placement_policy, is_ha)
         self.set_spot_instance_settings(maximum_bid_price_percentage, timeout_for_request, maximum_spot_instance_percentage)
         self.set_stable_spot_instance_settings(stable_maximum_bid_price_percentage, stable_timeout_for_request, stable_allow_fallback)
         self.set_ebs_volume_settings(ebs_volume_count, ebs_volume_type, ebs_volume_size)
@@ -1069,20 +1089,22 @@ class ClusterInfoV13():
         self.set_presto_settings(enable_presto, presto_custom_config)
 
     def set_ec2_settings(self,
-                           aws_access_key_id=None,
-                           aws_secret_access_key=None,
-                           aws_region=None,
-                           aws_availability_zone=None,
-                           vpc_id=None,
-                           subnet_id=None,
-                           bastion_node_public_dns=None,
-                           role_instance_profile=None):
+                         aws_access_key_id=None,
+                         aws_secret_access_key=None,
+                         aws_region=None,
+                         aws_availability_zone=None,
+                         vpc_id=None,
+                         subnet_id=None,
+                         master_elastic_ip=None,
+                         bastion_node_public_dns=None,
+                         role_instance_profile=None):
         self.ec2_settings['compute_access_key'] = aws_access_key_id
         self.ec2_settings['compute_secret_key'] = aws_secret_access_key
         self.ec2_settings['aws_region'] = aws_region
         self.ec2_settings['aws_preferred_availability_zone'] = aws_availability_zone
         self.ec2_settings['vpc_id'] = vpc_id
         self.ec2_settings['subnet_id'] = subnet_id
+        self.ec2_settings['master_elastic_ip'] = master_elastic_ip
         self.ec2_settings['bastion_node_public_dns'] = bastion_node_public_dns
         self.ec2_settings['role_instance_profile'] = role_instance_profile
 
@@ -1104,12 +1126,14 @@ class ClusterInfoV13():
                             custom_ec2_tags=None,
                             use_hadoop2=None,
                             use_spark=None,
-                            use_qubole_placement_policy=None,):
+                            use_qubole_placement_policy=None,
+                            is_ha=None):
         self.hadoop_settings['custom_config'] = custom_config
         self.hadoop_settings['use_hbase'] = use_hbase
         self.hadoop_settings['use_hadoop2'] = use_hadoop2
         self.hadoop_settings['use_spark'] = use_spark
         self.hadoop_settings['use_qubole_placement_policy'] = use_qubole_placement_policy
+        self.hadoop_settings['is_ha'] = is_ha
 
         if custom_ec2_tags and custom_ec2_tags.strip():
             try:
@@ -1166,21 +1190,4 @@ class ClusterInfoV13():
         """
         payload_dict = self.__dict__
         payload_dict.pop("api_version", None)
-        return _make_minimal(payload_dict)
-
-
-def _make_minimal(dictionary):
-    """
-    This function removes all the keys whose value is either None or an empty
-    dictionary.
-    """
-    new_dict = {}
-    for key, value in dictionary.items():
-        if value is not None:
-            if isinstance(value, dict):
-                new_value = _make_minimal(value)
-                if new_value:
-                    new_dict[key] = new_value
-            else:
-                new_dict[key] = value
-    return new_dict
+        return util._make_minimal(payload_dict)
